@@ -38,7 +38,3 @@ QSet<Maliit::HandlerState> OverridePlugin::supportedStates() const
 {
     return allowedStates;
 }
-
-#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
-Q_EXPORT_PLUGIN2(cxxoverrideplugin, OverridePlugin)
-#endif
