@@ -27,10 +27,8 @@ class OverridePlugin: public QObject,
 {
     Q_OBJECT
     Q_INTERFACES(Maliit::Plugins::InputMethodPlugin)
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
     Q_PLUGIN_METADATA(IID  "org.maliit.examples.cxx.overrideplugin"
                       FILE "overrideplugin.json")
-#endif
 
 public:
     OverridePlugin();
